@@ -4,64 +4,78 @@ mod tests {
 
     #[test]
     fn log() {
-        let mut logger = Logger::new("main", std::io::stdout());
-        logger.options.time_unit = TimeUnit::Milliseconds;
-        let timer_name = "Small code block";
-        logger.timer_start(timer_name);
-        logger.info("Hello !");
-        logger.warn("Oups !");
-        logger.timer_log_and_reset(timer_name);
-        logger.error("Oh no !");
-        logger.timer_log_and_stop(timer_name);
+        let x = 40;
+        let mut logger = Logger::new("MAIN", std::io::stdout());
+        logger.options.time_unit = TimeUnit::Nanoseconds;
+        logger.info("We will be testing a fibonnaci function");
+        logger.info(format!("Starting fibonnaci({})", x));
+        logger.timer_start("fibonnaci");
+        let y = fibo(x);
+        logger.timer_log_and_stop("fibonnaci");
+        logger.info(format!("The result of the fibonnaci function until {} is : {}", x, y));
+    }
 
-        logger.timer_start("Pause test");
-        for _i in 0..1000 {
-            logger.timer_resume("Pause test");
-            std::thread::sleep(std::time::Duration::from_millis(1));
-            logger.timer_pause("Pause test");
-            std::thread::sleep(std::time::Duration::from_micros(1));
+    fn fibo(n: usize) -> usize {
+        fibo_alt(1, 1, n)
+    }
+
+    fn fibo_alt(a: usize, b: usize, n: usize) -> usize {
+        if n == 0 {
+            b
+        } else {
+            fibo_alt(b, a + b, n - 1)
         }
-
-        logger.timer_log_and_stop("Pause test");
     }
 }
 
-pub use logger::*;
-
-mod logger {
-    use std::{
-        collections::HashMap,
-        fmt::Display,
-        io::stderr,
-        time::{SystemTime, Duration},
-    };
-    use colored::*;
-    use chrono::Utc;
+pub mod logger {
+    use std::{borrow::BorrowMut, collections::{HashMap, LinkedList}, fmt::Display, io::stderr, time::{SystemTime, Duration}};
+    use crate::modules::{self, Module};
 
+    /// A modular and powerful logger
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use rusty_logger::logger::{Logger, TimeUnit};
+    ///
+    /// let mut logger = Logger::new("EXAMPLE", std::io::stdout());
+    ///
+    /// logger.options.time_unit = TimeUnit::Nanoseconds;
+    ///
+    /// logger.info("We will be timing a for loop");
+    /// logger.timer_start("for loop");
+    /// for i in 0..1000 {
+    ///     // Do something...
+    /// }
+    /// logger.timer_log_and_stop("for loop");
+    /// logger.info("The for loop ended successfully");
+    /// ```
     pub struct Logger<T: std::io::Write> {
-        name: String,
-        pub options: Options<T>,
+        pub options: Options,
         timer: Timer,
-    }
-
-    pub struct Colors {
-        pub name: Color,
-        pub info: Color,
-        pub warn: Color,
-        pub fail: Color,
-        pub time: Color,
-        pub timer: Color,
-    }
-
-    pub struct Options<T: std::io::Write> {
-        pub time: bool,
-        pub name: bool,
-        pub time_unit: TimeUnit,
-        pub colors: Option<Colors>,
+        modules : LinkedList<Box<dyn Module>>,
         output: T,
     }
 
-    pub struct Timer {
+    pub struct LogMessage<'a> {
+        pub content: &'a str,
+        pub msg_type: LogType,
+        pub options: &'a Options,
+    }
+
+    /// A struct that stores the core data of a logger
+    ///
+    /// You may use the default configuration or make
+    /// your own with the `homemade` function depending
+    /// on your needs
+    pub struct Options {
+        pub name: String,
+        pub time_unit: TimeUnit,
+        pub colors: bool,
+    }
+
+    struct Timer {
         timers: HashMap<String, (SystemTime, SystemTime, bool)>,
     }
 
@@ -77,36 +91,14 @@ mod logger {
         Nanoseconds,
         Microseconds,
         Milliseconds,
-        Seconds
     }
 
-    impl Colors {
-        pub fn new() -> Colors {
-            Colors {
-                name: Color::Blue,
-                info: Color::Green,
-                warn: Color::Yellow,
-                fail: Color::Red,
-                time: Color::Magenta,
-                timer: Color::Cyan,
-            }
-        }
-    }
-
-    impl Default for Colors {
-        fn default() -> Self {
-            Colors::new()
-        }
-    }
-
-    impl<T: std::io::Write> Options<T> {
-        pub fn new(stream: T) -> Options<T> {
+    impl Options {
+        pub fn new(s: &str) -> Options {
             Options {
-                time: false,
-                name: true,
+                name: s.to_string(),
+                colors: true,
                 time_unit: TimeUnit::Milliseconds,
-                colors: Some(Colors::new()),
-                output: stream,
             }
         }
     }
@@ -129,29 +121,15 @@ mod logger {
         }
 
         fn get(&self, msg: &str) -> Option<Duration> {
-            if let Some(time) = self.timers.get(msg) {
-                Some(time.0.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
-            } else {
-                None
-            }
+            self.timers.get(msg).map(|time| time.0.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
         }
 
         fn stop(&mut self, msg: &str) -> Option<Duration> {
-            if let Some(time) = self.timers.remove(msg) {
-                Some(time.0.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
-            } else {
-                None
-            }
+            self.timers.remove(msg).map(|time| time.0.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
         }
 
         fn reset(&mut self, msg: &str) -> Option<Duration> {
-            if let Some(time) =
-                self.timers.insert(msg.to_string(), (SystemTime::now(), SystemTime::now(), false))
-            {
-                Some(time.0.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
-            } else {
-                None
-            }
+            self.timers.insert(msg.to_string(), (SystemTime::now(), SystemTime::now(), false)).map(|time| time.0.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
         }
 
         fn pause(&mut self, msg: &str) -> Option<Duration> {
@@ -172,7 +150,7 @@ mod logger {
             }
         }
 
-        fn resume(&mut self, msg: &str) {
+        fn resume(&mut self, msg: &str) -> Option<Duration>{
             if self.timers.contains_key(msg) {
                 let timer = self.timers.entry(msg.to_string()).or_insert((
                     SystemTime::now(),
@@ -186,6 +164,9 @@ mod logger {
                         .checked_add(timer.1.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
                         .unwrap_or_else(SystemTime::now);
                 }
+                Some(timer.0.elapsed().unwrap_or_else(|_| Duration::from_micros(0)))
+            } else {
+                None
             }
         }
     }
@@ -202,98 +183,68 @@ mod logger {
         /// logger.info("This is a new logger named 'name' !");
         /// ```
         pub fn new(s: &str, output: T) -> Self {
+            let mut modules: LinkedList<Box<dyn Module>> = LinkedList::new();
+            modules.push_back(Box::new(modules::Name{}));
+            modules.push_back(Box::new(modules::Time{}));
+            modules.push_back(Box::new(modules::Type{}));
             Logger {
-                name: s.to_string(),
-                options: Options::new(output),
+                options: Options::new(s),
                 timer: Timer::new(),
+                modules,
+                output,
             }
         }
 
-        /// Creates a new Logger with default values, custom options and the given name
+        /// Creates a new Logger with custom options and modules
         ///
         /// # Example
         ///
         /// ```
         /// use rusty_logger::logger::{Logger, Options};
+        /// use rusty_logger::modules::{self, Module};
+        /// use std::collections::LinkedList;
         ///
-        /// let options = Options::new(std::io::stdout());
-        /// let mut logger = Logger::with_options("name", options);
+        /// let options = Options::new("homemade logger");
+        /// 
+        /// let mut modules: LinkedList<Box<dyn Module>> = LinkedList::new();
+        /// modules.push_back(Box::new(modules::Name{}));
+        /// modules.push_back(Box::new(modules::Time{}));
+        /// modules.push_back(Box::new(modules::Type{}));
+        ///
+        /// let mut logger = Logger::homemade(std::io::stdout(), options, modules);
+        /// 
         /// logger.info("This is a new logger named 'name' which has 'options' as its options !");
         /// ```
-        pub fn with_options(s: &str, options: Options<T>) -> Logger<T> {
+        pub fn homemade(output: T, options: Options, modules: LinkedList<Box<dyn Module>>) -> Logger<T> {
             Logger {
-                name: s.to_string(),
                 options,
                 timer: Timer::new(),
+                modules,
+                output,
             }
         }
 
         /// Formats the message with the current logger options
         /// and then prints it. It is a private function and is
-        /// only used by the other functions (info, warn and
-        /// error) to print the message.
+        /// only used by the other internal functions to print
+        /// the message.
         ///
         /// It goes through the options of the logger it is
         /// called on and formats the message accordingly.
-        ///
-        /// It is also responsible to print everything in
-        /// the correct colors
-        fn log<D: Display>(&mut self, msg: D, msg_type: LogType) {
-            let mut output;
+        fn log(&mut self, msg: String, msg_type: LogType) {
+            let mut output = String::new();
 
-            if let Some(colors) = &self.options.colors {
-                match msg_type {
-                    LogType::Error => {
-                        output = format!("[{}]", "FAIL".color(colors.fail));
-                    }
-                    LogType::Warning => {
-                        output = format!("[{}]", "WARN".color(colors.warn));
-                    }
-                    LogType::Info => {
-                        output = format!("[{}]", "INFO".color(colors.info));
-                    }
-                    LogType::Time => {
-                        output = format!("[{}]", "TIME".color(colors.timer));
-                    }
-                };
+            let lm = LogMessage {
+                content: &msg[..],
+                msg_type,
+                options: &self.options,
+            };
 
-                if self.options.time {
-                    output = format!(
-                        "{}[{}]",
-                        output,
-                        Utc::now().format("%T").to_string().color(colors.time)
-                    );
-                }
-
-                if self.options.name {
-                    output = format!("[{}]{}", self.name.color(colors.name), output);
-                }
-            } else {
-                match msg_type {
-                    LogType::Error => {
-                        output = format!("[{}]", "FAIL");
-                    }
-                    LogType::Warning => {
-                        output = format!("[{}]", "WARN");
-                    }
-                    LogType::Info => {
-                        output = format!("[{}]", "INFO");
-                    }
-                    LogType::Time => {
-                        output = format!("[{}]", "TIME");
-                    }
-                };
-
-                if self.options.time {
-                    output = format!("{}[{}]", output, Utc::now().format("%T").to_string());
-                }
-
-                if self.options.name {
-                    output = format!("[{}]{}", self.name, output);
-                }
+            for module in self.modules.borrow_mut() {
+                output = format!("{}{}", output, module.print(&lm));
             }
 
-            match writeln!(self.options.output, "{}: {}", output, msg) {
+            match writeln!(self.output, "{}: {}", output, msg) {
                 std::result::Result::Err(err) => Logger::static_log(err, LogType::Error, stderr()),
                 std::result::Result::Ok(_) => {}
             }
@@ -313,7 +264,7 @@ mod logger {
         /// ```
         pub fn static_log<D: Display>(msg: D, msg_type: LogType, output: T) {
             let mut logger = Logger::new("STATIC", output);
-            logger.log(msg, msg_type);
+            logger.log(msg.to_string(), msg_type);
         }
 
         /// Logs the given message to the output
@@ -328,7 +279,7 @@ mod logger {
         /// logger.info("Here is some information I want to log");
         /// ```
         pub fn info<D: Display>(&mut self, msg: D) {
-            self.log(msg, LogType::Info);
+            self.log(msg.to_string(), LogType::Info);
         }
 
         /// Logs the given message to the output
@@ -343,7 +294,7 @@ mod logger {
         /// logger.warn("Here is a warning");
         /// ```
         pub fn warn<D: Display>(&mut self, msg: D) {
-            self.log(msg, LogType::Warning);
+            self.log(msg.to_string(), LogType::Warning);
         }
 
         /// Logs the given message to the output
@@ -358,7 +309,7 @@ mod logger {
         /// logger.info("Some error happened");
         /// ```
         pub fn error<D: Display>(&mut self, msg: D) {
-            self.log(msg, LogType::Error);
+            self.log(msg.to_string(), LogType::Error);
         }
 
         /// Starts a new timer with the given name
@@ -394,7 +345,7 @@ mod logger {
         }
 
         /// Resumes the timer with the given name
-        pub fn timer_resume(&mut self, msg: &str) {
+        pub fn timer_resume(&mut self, msg: &str) -> Option<Duration> {
             self.timer.resume(msg)
         }
 
@@ -415,7 +366,7 @@ mod logger {
         ///
         /// logger.timer_log("new_timer");
         /// ```
-        pub fn timer_log(&mut self, msg: &str) {
+        pub fn timer_log(&mut self, msg: &str) -> Option<Duration> {
             if let Some(time) = self.timer.get(msg) {
                 match self.options.time_unit {
                     TimeUnit::Nanoseconds => {
@@ -427,12 +378,11 @@ mod logger {
                     TimeUnit::Milliseconds => {
                         self.log(format!("{} - {}ms", msg, time.as_millis()), LogType::Time);
                     }
-                    TimeUnit::Seconds => {
-                        self.log(format!("{} - {}ms", msg, time.as_secs()), LogType::Time);
-                    }
                 }
+                Some(time)
             } else {
                 Logger::static_log(msg, LogType::Error, stderr());
+                None
             }
         }
 
@@ -453,9 +403,9 @@ mod logger {
         ///
         /// logger.timer_log_and_stop("new_timer");
         /// ```
-        pub fn timer_log_and_stop(&mut self, msg: &str) {
+        pub fn timer_log_and_stop(&mut self, msg: &str) -> Option<Duration> {
             self.timer_log(msg);
-            self.timer_stop(msg);
+            self.timer_stop(msg)
         }
 
         /// Logs the time elapsed between the
@@ -479,9 +429,83 @@ mod logger {
         ///
         /// logger.timer_log_and_stop("new_timer");
         /// ```
-        pub fn timer_log_and_reset(&mut self, msg: &str) {
+        pub fn timer_log_and_reset(&mut self, msg: &str) -> Option<Duration> {
             self.timer_log(msg);
-            self.timer_reset(msg);
+            self.timer_reset(msg)
+        }
+    }
+}
+
+pub mod modules {
+    use crate::logger::{LogType, LogMessage};
+    use colored::{Color, Colorize};
+    use chrono::Utc;
+
+    /// Trait that allows your struct to be implemented
+    /// as a logger module
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use rusty_logger::modules::Module;
+    /// use rusty_logger::logger::LogMessage;
+    ///
+    /// struct IncrementalModule {n: usize};
+    ///
+    /// impl Module for IncrementalModule {
+    ///     fn print(&mut self, msg: &LogMessage) -> String {
+    ///         self.n = self.n + 1;
+    ///         (self.n - 1).to_string()
+    ///     }
+    /// }
+    /// ```
+    pub trait Module {
+        fn print(&mut self, msg: &LogMessage) -> String;
+    }
+
+    /// A simple module to display the name of the logger
+    pub struct Name;
+
+    impl Module for Name {
+        fn print(&mut self, msg: &LogMessage) -> String {
+            format!("[{}]", msg.options.name.color(Color::Blue))
+        }
+    }
+    
+    /// A simple module to display the type of the message
+    pub struct Type;
+
+    impl Module for Type {
+        fn print(&mut self, msg: &LogMessage) -> String {
+            let s = if msg.options.colors {
+                match msg.msg_type {
+                    LogType::Error => "ERROR".color(Color::Red),
+                    LogType::Warning => "WARN".color(Color::Yellow),
+                    LogType::Info => "INFO".color(Color::Green),
+                    LogType::Time => "TIME".color(Color::Cyan),
+                }.to_string()
+            } else {
+                match msg.msg_type {
+                    LogType::Error => "ERROR",
+                    LogType::Warning => "WARN",
+                    LogType::Info => "INFO",
+                    LogType::Time => "TIME",
+                }.to_string()
+            };
+            format!("[{}]", s)
+        }
+    }
+
+    /// A simple module to display the time of the log
+    pub struct Time;
+
+    impl Module for Time {
+        fn print(&mut self, msg: &LogMessage) -> String {
+            if msg.options.colors {
+                format!("[{}]", Utc::now().format("%T").to_string().color(Color::Magenta))
+            } else {
+                format!("[{}]", Utc::now().format("%T").to_string())
+            }
         }
     }
 }
